from threading import Thread

from je_editor.pyside_ui.main_ui.ai_widget.ai_config import ai_config
from je_editor.pyside_ui.main_ui.ai_widget.langchain_interface import LangChainInterface


class AskThread(Thread):

    def __init__(self, lang_chain_interface: LangChainInterface, prompt):
        super().__init__()
        self.lang_chain_interface = lang_chain_interface
        self.prompt = prompt


    def run(self):
        ai_response = self.lang_chain_interface.call_ai_model(prompt=self.prompt)
        ai_config.message_queue.put(ai_response)


from PySide6.QtWidgets import QWidget, QLineEdit, QPushButton, QMessageBox, QGridLayout, QLabel

from je_editor.pyside_ui.main_ui.ai_widget.ai_config import ai_config
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class SetAIDialog(QWidget):

    def __init__(self):
        jeditor_logger.info("Init SetAIDialog")
        super().__init__()
        self.base_url_label = QLabel(language_wrapper.language_word_dict.get("base_url_label"))
        self.base_url_input = QLineEdit()
        self.api_key_label = QLabel(language_wrapper.language_word_dict.get("api_key_label"))
        self.api_key_input = QLineEdit()
        self.chat_model_label = QLabel(language_wrapper.language_word_dict.get("ai_model_label"))
        self.chat_model_input = QLineEdit()
        self.add_ai_info_button = QPushButton()
        self.add_ai_info_button.setText(language_wrapper.language_word_dict.get("add_ai_model_pushbutton"))
        self.add_ai_info_button.clicked.connect(self.update_ai_config)
        self.grid_layout = QGridLayout()
        self.grid_layout.addWidget(self.base_url_label, 0, 0)
        self.grid_layout.addWidget(self.base_url_input, 0, 1)
        self.grid_layout.addWidget(self.api_key_label, 1, 0)
        self.grid_layout.addWidget(self.api_key_input, 1, 1)
        self.grid_layout.addWidget(self.chat_model_label, 2, 0)
        self.grid_layout.addWidget(self.chat_model_input, 2, 1)
        self.grid_layout.addWidget(self.add_ai_info_button, 3, 1)
        self.setWindowTitle(language_wrapper.language_word_dict.get("add_ai_model_title"))
        self.setLayout(self.grid_layout)

    def update_ai_config(self):
        base_url = self.base_url_input.text().strip()
        api_key = self.api_key_input.text().strip()
        chat_model = self.chat_model_input.text().strip()
        if base_url and chat_model:
            ai_config.choosable_ai.update(
                {"AI_model": {"base_url": base_url, "api_key": api_key, "chat_model": chat_model}}
            )
        else:
            QMessageBox.warning(self,
                                language_wrapper.language_word_dict.get("set_ai_model_warring_title"),
                                language_wrapper.language_word_dict.get("set_ai_model_warring_text"))



from queue import Queue


class AIConfig(object):

    def __init__(self):
        self.current_ai_model_system_prompt: str = ""
        self.choosable_ai: dict[str, dict[str, str]] = {
            "AI_model": {
                "ai_base_url": "",
                "ai_api_key": "",
                "chat_model": "",
                "prompt_template": "",
            }
        }
        self.message_queue = Queue()


ai_config = AIConfig()



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Union

from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QFontDatabase
from PySide6.QtWidgets import QWidget, QPlainTextEdit, QScrollArea, QLabel, QComboBox, QGridLayout, QPushButton, \
    QMessageBox, QSizePolicy, QLineEdit

from je_editor.pyside_ui.dialog.ai_dialog.set_ai_dialog import SetAIDialog
from je_editor.pyside_ui.main_ui.ai_widget.ai_config import AIConfig, ai_config
from je_editor.pyside_ui.main_ui.ai_widget.ask_thread import AskThread
from je_editor.pyside_ui.main_ui.ai_widget.langchain_interface import LangChainInterface
from je_editor.utils.json.json_file import read_json
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain


class ChatUI(QWidget):

    def __init__(self, main_window: EditorMain):
        super().__init__()
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.main_window = main_window
        # Chat panel
        self.chat_panel = QPlainTextEdit()
        self.chat_panel.setLineWrapMode(self.chat_panel.LineWrapMode.NoWrap)
        self.chat_panel.setReadOnly(True)
        self.chat_panel_scroll_area = QScrollArea()
        self.chat_panel_scroll_area.setWidgetResizable(True)
        self.chat_panel_scroll_area.setViewportMargins(0, 0, 0, 0)
        self.chat_panel_scroll_area.setWidget(self.chat_panel)
        self.chat_panel.setFont(QFontDatabase.font(self.font().family(), "", 16))
        # Prompt input
        self.prompt_input = QLineEdit()
        self.prompt_input.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)
        self.prompt_input.returnPressed.connect(self.call_ai_model)
        # Font size combobox
        self.font_size_label = QLabel(language_wrapper.language_word_dict.get("font_size"))
        self.font_size_combobox = QComboBox()
        for font_size in range(2, 101, 2):
            self.font_size_combobox.addItem(str(font_size))
        self.font_size_combobox.setCurrentText("16")
        self.font_size_combobox.currentTextChanged.connect(self.update_panel_text_size)
        # Buttons
        self.set_ai_config_button = QPushButton(language_wrapper.language_word_dict.get("chat_ui_set_ai_button"))
        self.set_ai_config_button.clicked.connect(self.set_ai_config)
        self.load_ai_config_button = QPushButton(language_wrapper.language_word_dict.get("chat_ui_load_ai_button"))
        self.load_ai_config_button.clicked.connect(lambda: self.load_ai_config(show_load_complete=True))
        self.call_ai_model_button = QPushButton(language_wrapper.language_word_dict.get("chat_ui_call_ai_model_button"))
        self.call_ai_model_button.clicked.connect(self.call_ai_model)
        # Add to layout
        self.grid_layout = QGridLayout()
        self.grid_layout.addWidget(self.chat_panel_scroll_area, 0, 0, 1, 4)
        self.grid_layout.addWidget(self.call_ai_model_button, 1, 0)
        self.grid_layout.addWidget(self.font_size_combobox, 1, 1)
        self.grid_layout.addWidget(self.set_ai_config_button, 1, 2)
        self.grid_layout.addWidget(self.load_ai_config_button, 1, 3)
        self.grid_layout.addWidget(self.prompt_input, 2, 0, 1, 4)

        # Variable
        self.ai_config: AIConfig = ai_config
        self.lang_chain_interface: Union[LangChainInterface, None] = None
        self.set_ai_config_dialog = None
        # Timer to pop queue
        self.pull_message_timer = QTimer(self)
        self.pull_message_timer.setInterval(1000)
        self.pull_message_timer.timeout.connect(self.pull_message)
        self.pull_message_timer.start()

        # Set layout
        self.setLayout(self.grid_layout)

        self.load_ai_config()

    def update_panel_text_size(self):
        self.chat_panel.setFont(
            QFontDatabase.font(self.font().family(), "", int(self.font_size_combobox.currentText())))

    def load_ai_config(self, show_load_complete: bool = False):
        ai_config_file = Path(str(Path.cwd()) + "/" + ".jeditor/ai_config.json")
        if ai_config_file.exists():
            with open(ai_config_file, "r", encoding="utf-8"):
                json_data: dict = read_json(str(ai_config_file))
            if json_data:
                if json_data.get("AI_model") and len(json_data.get("AI_model")) == 4:
                    ai_info: dict = json_data.get("AI_model")
                    if ai_info.get("ai_base_url") and ai_info.get("chat_model"):
                        ai_config.choosable_ai.update(json_data)
                    self.lang_chain_interface = LangChainInterface(
                        main_window=self,
                        api_key=ai_info.get("ai_api_key"),
                        base_url=ai_info.get("ai_base_url"),
                        chat_model=ai_info.get("chat_model"),
                        prompt_template=ai_info.get("prompt_template"),
                    )
            if show_load_complete:
                load_complete = QMessageBox(self)
                load_complete.setWindowTitle(language_wrapper.language_word_dict.get("load_ai_messagebox_title"))
                load_complete.setText(language_wrapper.language_word_dict.get("load_ai_messagebox_text"))
                load_complete.exec()


    def call_ai_model(self):
        if isinstance(self.lang_chain_interface, LangChainInterface):
            thread = AskThread(lang_chain_interface=self.lang_chain_interface, prompt=self.prompt_input.text())
            thread.start()
        else:
            ai_info = ai_config.choosable_ai.get('AI_model')
            QMessageBox.warning(self,
                                language_wrapper.language_word_dict.get("call_ai_model_error_title"),
                                language_wrapper.language_word_dict.get(
                                    f"ai_api_key: {ai_info.get('ai_api_key')}, \n"
                                    f"ai_base_url: {ai_info.get('ai_base_url')}, \n"
                                    f"chat_model: {ai_info.get('chat_model')}, \n"
                                    f"prompt_template: {ai_info.get('prompt_template')}"))

    def pull_message(self):
        if not ai_config.message_queue.empty():
            ai_response = ai_config.message_queue.get_nowait()
            self.chat_panel.appendPlainText(ai_response)
            self.chat_panel.appendPlainText("\n")

    def set_ai_config(self):
        # Set and output AI a config file
        self.set_ai_config_dialog = SetAIDialog()
        self.set_ai_config_dialog.show()


from __future__ import annotations

import os
import re
from typing import Union, TYPE_CHECKING

from PySide6.QtWidgets import QMessageBox
from langchain.prompts.chat import SystemMessagePromptTemplate
from langchain_openai import ChatOpenAI
from pydantic import SecretStr

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.ai_widget.chat_ui import ChatUI

class LangChainInterface(object):

    def __init__(self, main_window: ChatUI, prompt_template: str, base_url: str, api_key: Union[SecretStr, str],
                 chat_model: str):
        self.system_message_prompt = SystemMessagePromptTemplate.from_template(prompt_template)
        self.base_url = base_url
        self.api_key = api_key
        self.chat_model = chat_model
        self.main_window = main_window
        os.environ["OPENAI_BASE_URL"] = self.base_url
        os.environ["OPENAI_API_KEY"] = self.api_key
        os.environ["CHAT_MODEL"] = self.chat_model
        self.chat_ai = ChatOpenAI(base_url=self.base_url, api_key=self.api_key, model=self.chat_model)

    def call_ai_model(self, prompt: str) -> str | None:
        message = None
        try:
            message = self.chat_ai.invoke(prompt).text()
            match = re.search(r"</think>\s*(.*)", message, re.DOTALL)
            if match:
                message = match.group(1).strip()
            else:
                message = message

        except Exception as error:
            QMessageBox.warning(self.main_window,
                                language_wrapper.language_word_dict.get("call_ai_model_error_title"),
                                str(error))
        return message
